@@ -28,24 +28,25 @@
 #endif
 
 #include <gtkmozembed.h>
-#include "ui_htmlview.h"
+#include "ui/ui_htmlview.h"
 #include "mozembed.h"
+#include "plugin.h"
 
 static void mozilla_init(void) {
 
 	/* Avoid influencing the component loading by $MOZILLA_FIVE_HOME */
 	g_unsetenv("MOZILLA_FIVE_HOME");
-	
+
 	gtk_moz_embed_set_comp_path(MOZILLA_LIB_ROOT);
 
 	mozembed_init();	
 }
 
-static htmlviewPluginInfo mozillaInfo = {
-	.api_version	= HTMLVIEW_API_VERSION,
+static struct htmlviewPlugin mozillaInfo = {
+	.api_version	= HTMLVIEW_PLUGIN_API_VERSION,
 	.name		= "Mozilla",
-	.init		= mozilla_init,
-	.deinit		= mozembed_deinit,
+	.plugin_init	= mozilla_init,
+	.plugin_deinit	= mozembed_deinit,
 	.create		= mozembed_create,
 	.write		= mozembed_write,
 	.launch		= mozembed_launch_url,
@@ -53,8 +54,17 @@ static htmlviewPluginInfo mozillaInfo = {
 	.zoomLevelGet	= mozsupport_get_zoom,
 	.zoomLevelSet	= mozsupport_set_zoom,
 	.scrollPagedown	= mozsupport_scroll_pagedown,
-	.setProxy	= mozembed_set_proxy
+	.setProxy	= mozembed_set_proxy,
+	.setOffLine	= mozsupport_set_offline_mode
 };
 
+static struct plugin pi = {
+	PLUGIN_API_VERSION,
+	"Mozilla Rendering Plugin",
+	PLUGIN_TYPE_HTML_RENDERER,
+	&mozillaInfo
+};
+
+DECLARE_PLUGIN(pi);
 DECLARE_HTMLVIEW_PLUGIN(mozillaInfo);
 
